package hk.hku.eee.cro.cro;

import java.util.Random;

/**
 * This class is a sample of how to use CRO in solving optimization problems.
 * Since the benchmark set is a collection of continuous optimization problems,
 * we employ Real-Coded CRO to solve these problems. RCCRO has the same
 * framework with CRO and you can make the codes solve combinatorial problems
 * with very little modifications.
 *
 * @author James
 */
public class RCCRO {

    /**
     * This is the total number of repeats expected to perform.
     */
    private static final int NUMREPEATCOUNT = 25;

    /**
     * This is the entry point of the code. You can see the flow of how to initialize
     * a problem and create a CRO container to solve it.
     * @param args Useless.
     */
    public static void main(String[] args){
        Util.r = new Random();
        // We have 23 functions to solve. Iteration.
        for (int problemIndex = 1; problemIndex <= 23; problemIndex++) {
            System.out.println("==============================");
            Problem problem = new Problem(problemIndex);
            double globals[] = new double[NUMREPEATCOUNT];
            double times[] = new double[NUMREPEATCOUNT];
            // For every function, we perform NUMREPEATCOUNT repeats.
            for (int i = 0; i < NUMREPEATCOUNT; i++) {
                long start = System.currentTimeMillis();
                Container container = new Container(problem);
                problem.initContainer(container);
                globals[i] = container.run();
                times[i] = (System.currentTimeMillis() - start) / 1000F;
                System.out.println(globals[i]);
            }
            System.gc();
        }
    }
}
